import 'package:flutter/material.dart';
import 'package:flutter_keyboard_visibility/flutter_keyboard_visibility.dart';
import 'package:getx_extended_dart/controller/task_controller.dart';
import 'package:getx_extended_dart/export.dart';

class QuickRoot extends StatelessWidget {
  Widget child;
  Widget keyboardToolbar;
  Widget loading;
  Future Function(dynamic error) consumeError;

  QuickRoot({
    required this.child,
    required this.keyboardToolbar,
    required this.loading,
    required this.consumeError,
  });

  late TaskController _task = Get.find();

  @override
  Widget build(BuildContext context) => Material(
        child: SafeArea(
          child: Obx(
            () => KeyboardVisibilityBuilder(
              builder: (_, isKeyboardVisible) {
                _checkError();
                return _body();
              },
            ),
          ),
        ),
      );

  void _checkError() {
    final error = _task.errors.firstOrNull;
    if (error == null) return;
    WidgetsBinding.instance?.addPostFrameCallback((timeStamp) async {
      await consumeError(error);
      _task.dismiss(error);
    });
  }

  Widget _body() => Column(
        children: [
          Expanded(
            child: Stack(
              children: [
                child,
                ObxValue<RxBool>(
                  (rx) => rx.value ? loading : Container(),
                  _task.isProcessing,
                )
              ],
            ),
          ),
          keyboardToolbar!
        ],
      );
}
